@@ -26,11 +26,12 @@ Runtime checks.
 import gtk
 import gst
 import sys
-import instance
-from configure import APPNAME, PYGTK_REQ, PYGST_REQ, GST_REQ, GNONLIN_REQ, PYCAIRO_REQ
 
 from gettext import gettext as _
 
+from pitivi.instance import PiTiVi
+from pitivi.configure import APPNAME, PYGTK_REQ, PYGST_REQ, GST_REQ, GNONLIN_REQ, PYCAIRO_REQ
+
 def initial_checks():
     """
     do some basic check
@@ -99,8 +100,8 @@ def check_required_version(modulename):
         if list(gtk.pygtk_version) < _string_to_list(PYGTK_REQ):
             return [PYGTK_REQ, _version_to_string(gtk.pygtk_version)]
     if modulename == "pygst":
-        if list(gst.pygst_version) < _string_to_list(PYGST_REQ):
-            return [PYGST_REQ, _version_to_string(gst.pygst_version)]
+        if list(gst.get_pygst_version()) < _string_to_list(PYGST_REQ):
+            return [PYGST_REQ, _version_to_string(gst.get_pygst_version())]
     if modulename == "cairo":
         import cairo
         if _string_to_list(cairo.cairo_version_string()) < _string_to_list(PYCAIRO_REQ):
@@ -116,7 +117,7 @@ def check_required_version(modulename):
 
 def _checks():
     reg = gst.registry_get_default()
-    if instance.PiTiVi:
+    if PiTiVi:
         return (_("%s is already running!") % APPNAME,
                 _("An instance of %s is already running in this script.") % APPNAME)
     if not reg.find_plugin("gnonlin"):
@@ -127,7 +128,7 @@ def _checks():
                 _("Make sure you have installed gst-plugins-good and is available in the GStreamer plugin path."))
     if not hasattr(gtk.gdk.Window, 'cairo_create'):
         return (_("PyGTK doesn't have Cairo support!"),
-                _("Please use a version of the Python bindings for GTK+ built with Cairo support."))
+                _("Please use a version of the GTK+ Python Bindings built with Cairo support."))
     if not initiate_videosinks():
         return (_("Could not initiate the video output plugins"),
                 _("Make sure you have at least one valid video output sink available (xvimagesink or ximagesink)"))
@@ -135,14 +136,14 @@ def _checks():
         return (_("Could not initiate the audio output plugins"),
                 _("Make sure you have at least one valid audio output sink available (alsasink or osssink)"))
     if not __try_import__("cairo"):
-        return (_("Could not import the Python bindings for cairo"),
-                _("Make sure you have the Python bindings for cairo installed"))
+        return (_("Could not import the Cairo Python Bindings"),
+                _("Make sure you have the Cairo Python Bindings installed"))
     if not __try_import__("gtk.glade"):
-        return (_("Could not import the Python bindings for libglade"),
-                _("Make sure you have the Python bindings for libglade installed"))
+        return (_("Could not import the libglade Python Bindings"),
+                _("Make sure you have the libglade Python bindings installed"))
     if not __try_import__("goocanvas"):
-        return (_("Could not import the Python bindings for goocanvas"),
-                _("Make sure you have the Python bindings for goocanvas installed"))
+        return (_("Could not import the goocanvas Python Bindings"),
+                _("Make sure you have the goocanvas Python Bindings installed"))
     req, inst = check_required_version("gtk")
     if req:
         return (_("You do not have a recent enough version of the GTK+ Python Bindings (currently %s)") % inst,
