@@ -23,23 +23,33 @@
 Widget for the output settings
 """
 
-import gobject
 import gtk
+import gst
+from pitivi.log.loggable import Loggable
+from pitivi.encode import encoders_muxer_compatible, muxer_can_sink_raw_audio, muxer_can_sink_raw_video
 from glade import GladeWidget
-from pitivi.settings import encoders_muxer_compatible
 from gstwidget import GstElementSettingsDialog
 
-class ExportSettingsWidget(GladeWidget):
+from gettext import gettext as _
+
+class ExportSettingsWidget(GladeWidget, Loggable):
     glade_file = "exportsettingswidget.glade"
-    video_presets = ( ("DVD PAL",  720,    576,    25.0),
-                      ("320x240x30", 320,  240,    30.0) )
-    video_rates = ( ("12 fps",      12.0),
-                    ("24 fps",      24.0),
-                    ("23,97 fps",   2400.0/1001.0),
-                    ("25 fps",      25.0),
-                    ("29,97 fps",   3000.0/1001.0),
-                    ("30 fps",      30.0),
-                    ("60 fps",      60.0) )
+    video_presets = ((_("576p (PAL DV/DVD)"), 720, 576, 25.0, 1.0),
+                    (_("480p (NTSC DV/DVD)"), 720, 480, 30000.0, 1001.0),
+                    (_("720p HD"), 1280, 720, 30000.0, 1001.0),
+                    (_("1080p full HD"), 1920, 1080, 30000.0, 1001.0),
+                    (_("QVGA (320x240)"), 320, 240, 30.0, 1.0),
+                    (_("VGA (640x480)"), 640, 480, 30.0, 1.0),
+                    (_("SVGA (800x600)"), 800, 600, 30.0, 1.0),
+                    (_("WXGA (1024x768)"), 1024, 768, 30.0, 1.0),
+                    )
+    video_rates = ( ("12 fps",      12.0, 1.0),
+                    ("24 fps",      24.0, 1.0),
+                    ("23,97 fps",   24000.0, 1001.0),
+                    ("25 fps",      25.0, 1.0),
+                    ("29,97 fps",   30000.0, 1001.0),
+                    ("30 fps",      30.0, 1.0),
+                    ("60 fps",      60.0, 1.0) )
     audio_presets = ( ("CD",  2,      44100,  16), )
     audio_rates = ( ("8 KHz",   8000),
                     ("11 KHz",  11025),
@@ -51,18 +61,25 @@ class ExportSettingsWidget(GladeWidget):
                      ("16 bit", 16),
                      ("24 bit", 24),
                      ("32 bit", 32) )
-    
+
 
     def __init__(self):
         GladeWidget.__init__(self)
+        Loggable.__init__(self)
         self.settings = None
         self.validaencoders = []
         self.validvencoders = []
+        # cached values
         self.containersettings = {}
         self.vcodecsettings = {}
         self.acodecsettings = {}
+        self.muxer = None
+        self.vencoder = None
+        self.aencoder = None
+        self._loading = False
 
     def setSettings(self, settings):
+        self.debug("settings:%s", settings)
         self.settings = settings
         self._fillSettings()
 
@@ -77,15 +94,19 @@ class ExportSettingsWidget(GladeWidget):
         value = -1
         for rate in self.video_rates:
             videolist.append([rate[0]])
-            if rate[1] == self.settings.videorate:
+            if rate[1] == self.settings.videorate.num and rate[2] == self.settings.videorate.denom:
                 value = idx
             idx = idx + 1
         self.videoratecbox.set_active(value)
 
         videolist = self.videocombobox.get_model()
         videolist.clear()
+        fps_map = dict(((num, denom), desc)
+                for desc, num, denom in self.video_rates)
         for preset in self.video_presets:
-            videolist.append([preset[0]])
+            fps = preset[3:5]
+            videolist.append(["%s - %s" % (preset[0], fps_map[fps])])
+
         videolist.append(["Custom"])
 
         # find out from the project settings which combo to take
@@ -93,7 +114,8 @@ class ExportSettingsWidget(GladeWidget):
         for preset in self.video_presets:
             if (self.settings.videowidth,
                 self.settings.videoheight,
-                self.settings.videorate) == preset[1:]:
+                self.settings.videorate.num,
+                self.settings.videorate.denom) == preset[1:]:
                 break
             idx = idx + 1
         self.videocombobox.set_active(idx)
@@ -113,7 +135,7 @@ class ExportSettingsWidget(GladeWidget):
         for depth in self.audio_depths:
             audiolist.append([depth[0]])
         self._putGoodAudiodepth(self.settings.audiodepth)
-        
+
         audiolist = self.audiocombobox.get_model()
         audiolist.clear()
         for preset in self.audio_presets:
@@ -141,7 +163,7 @@ class ExportSettingsWidget(GladeWidget):
                 selected = idx
             idx = idx + 1
         self.vcodeccbox.set_active(selected)
-        
+
         aenclist = self.acodeccbox.get_model()
         aenclist.clear()
         idx = 0
@@ -152,8 +174,9 @@ class ExportSettingsWidget(GladeWidget):
                 selected = idx
             idx = idx + 1
         self.acodeccbox.set_active(selected)
-        
+
         # Muxer
+        self._loading = True
         self.muxers = self.settings.muxers
         muxs = self.muxercombobox.get_model()
         muxs.clear()
@@ -164,9 +187,13 @@ class ExportSettingsWidget(GladeWidget):
             if mux.get_name() == self.settings.muxer:
                 selected = idx
             idx = idx + 1
+        self._loading = False
         self.muxercombobox.set_active(selected)
 
         # Encoder/Muxer settings
+        self.muxer = self.settings.muxer
+        self.vencoder = self.settings.vencoder
+        self.aencoder = self.settings.aencoder
         self.containersettings = self.settings.containersettings
         self.acodecsettings = self.settings.acodecsettings
         self.vcodecsettings = self.settings.vcodecsettings
@@ -197,9 +224,11 @@ class ExportSettingsWidget(GladeWidget):
 
     def _videoComboboxChangedCb(self, widget):
         idx = widget.get_active()
-        if idx == len(self.video_presets):
+        if idx == -1 or idx == len(self.video_presets):
+            # not a preset
             activate = True
         else:
+            # valid preset
             activate = False
             self.videowidthspin.set_value(self.video_presets[idx][1])
             self.videoheightspin.set_value(self.video_presets[idx][2])
@@ -210,9 +239,11 @@ class ExportSettingsWidget(GladeWidget):
 
     def _audioComboboxChangedCb(self, widget):
         idx = widget.get_active()
-        if idx == len(self.audio_presets):
+        if idx == -1 or idx == len(self.audio_presets):
+            # not a preset
             activate = True
         else:
+            # valid preset
             activate = False
             self.audiochanncbox.set_active(self.audio_presets[idx][1] - 1)
             self._putGoodAudiorate(self.audio_presets[idx][2])
@@ -222,19 +253,33 @@ class ExportSettingsWidget(GladeWidget):
         self.audiodepthcbox.set_sensitive(activate)
 
     def _muxerComboboxChangedCb(self, widget):
+        if self._loading:
+            return
+        # get previous video encoder name
         if self.validvencoders:
-            prevvenc = self.validvencoders[self.vcodeccbox.get_active()].get_name()
+            vidx = self.vcodeccbox.get_active()
+            if vidx < len(self.validvencoders):
+                prevvenc = self.validvencoders[vidx].get_name()
+            elif vidx == len(self.validvencoders):
+                prevvenc = None
         else:
             prevvenc = self.settings.vencoder
+
+        # get previous audio encoder name
         if self.validaencoders:
-            prevaenc = self.validaencoders[self.acodeccbox.get_active()].get_name()
+            aidx = self.acodeccbox.get_active()
+            if aidx < len(self.validaencoders):
+                prevaenc = self.validaencoders[aidx].get_name()
+            elif aidx == len(self.validaencoders):
+                prevaenc = None
         else:
             prevaenc = self.settings.aencoder
         # find the valid audio/video codec with the given muxer
+        muxer = self.muxers[widget.get_active()]
         self.validaencoders = encoders_muxer_compatible(self.settings.aencoders,
-                                                        self.muxers[widget.get_active()])
+                                                        muxer)
         self.validvencoders = encoders_muxer_compatible(self.settings.vencoders,
-                                                        self.muxers[widget.get_active()])
+                                                        muxer)
 
         venclist = self.vcodeccbox.get_model()
         venclist.clear()
@@ -245,6 +290,10 @@ class ExportSettingsWidget(GladeWidget):
             if enc.get_name() == prevvenc:
                 selected = idx
             idx = idx + 1
+        if muxer_can_sink_raw_video(muxer):
+            venclist.append(["Raw Video"])
+            if prevvenc == None:
+                selected = idx
         self.vcodeccbox.set_active(selected)
 
         aenclist = self.acodeccbox.get_model()
@@ -256,48 +305,65 @@ class ExportSettingsWidget(GladeWidget):
             if enc.get_name() == prevaenc:
                 selected = idx
             idx = idx + 1
+        if muxer_can_sink_raw_audio(muxer):
+            aenclist.append(["Raw Audio"])
+            if prevaenc == None:
+                selected = idx
         self.acodeccbox.set_active(selected)
 
-    def runSettingsDialog(self, factory, settings):
+    def runSettingsDialog(self, factory, settings={}):
         dialog = GstElementSettingsDialog(factory, settings)
         if dialog.run() == gtk.RESPONSE_OK:
             dialog.hide()
-            settings = dialog.get_settings()
+            settings = dialog.getSettings()
         else:
             settings = None
         dialog.destroy()
-        return settings        
+        return settings
 
     def _muxerSettingsButtonClickedCb(self, button):
         factory = self.settings.muxers[self.muxercombobox.get_active()]
         if not factory:
             return
-        set = self.runSettingsDialog(factory, self.containersettings)
+        if factory.get_name() == self.settings.muxer:
+            set = self.runSettingsDialog(factory, self.containersettings)
+        else:
+            set = self.runSettingsDialog(factory)
         if set:
             self.containersettings = set
+            self.muxer = factory.get_name()
 
     def _acodecSettingsButtonClickedCb(self, button):
         factory = self.validaencoders[self.acodeccbox.get_active()]
         if not factory:
             return
-        set = self.runSettingsDialog(factory, self.acodecsettings)
+        if factory.get_name() == self.aencoder:
+            set = self.runSettingsDialog(factory, self.acodecsettings)
+        else:
+            set = self.runSettingsDialog(factory)
         if set:
             self.acodecsettings = set
+            self.aencoder = factory.get_name()
 
     def _vcodecSettingsButtonClickedCb(self, button):
         factory = self.validvencoders[self.vcodeccbox.get_active()]
         if not factory:
             return
-        settings = self.runSettingsDialog(factory, self.vcodecsettings)
+        if factory.get_name() == self.vencoder:
+            settings = self.runSettingsDialog(factory, self.vcodecsettings)
+        else:
+            settings = self.runSettingsDialog(factory)
         if settings:
             self.vcodecsettings = settings
-        
-        
+            self.vencoder = factory.get_name()
+
+
     def updateSettings(self):
+        """ Updates and returns the ExportSettings configured in the widget """
         # Video Settings
         width = self.videowidthspin.get_value()
         height = self.videoheightspin.get_value()
-        rate = self.video_rates[self.videoratecbox.get_active()][1]
+        rate = gst.Fraction(*self.video_rates[self.videoratecbox.get_active()][1:])
         self.settings.setVideoProperties(width, height, rate)
 
         # Audio Settings
@@ -308,21 +374,47 @@ class ExportSettingsWidget(GladeWidget):
 
         # Encoders
         muxer = self.settings.muxers[self.muxercombobox.get_active()].get_name()
-        vencoder = self.validvencoders[self.vcodeccbox.get_active()].get_name()
-        aencoder = self.validaencoders[self.acodeccbox.get_active()].get_name()
+        vidx = self.vcodeccbox.get_active()
+        if vidx < len(self.validvencoders):
+            vencoder = self.validvencoders[vidx].get_name()
+        elif vidx == len(self.validvencoders):
+            vencoder = None
+        else:
+            self.warning("we don't want any video stream")
+        aidx = self.acodeccbox.get_active()
+        if aidx < len(self.validaencoders):
+            aencoder = self.validaencoders[aidx].get_name()
+        elif aidx == len(self.validaencoders):
+            aencoder = None
+        else:
+            self.warning("we don't want any audio stream")
         self.settings.setEncoders(muxer, vencoder, aencoder)
 
         # encoder/muxer settings
-        self.settings.containersettings = self.containersettings
-        self.settings.acodecsettings = self.acodecsettings
-        self.settings.vcodecsettings = self.vcodecsettings
-        
+        # only store cached values if no different factory was chosen.
+        if muxer == self.muxer:
+            self.settings.containersettings = self.containersettings
+        else:
+            self.settings.containersettings = {}
+        if aencoder == self.aencoder:
+            self.settings.acodecsettings = self.acodecsettings
+        else:
+            self.settings.acodecsettings = {}
+        if vencoder == self.vencoder:
+            self.settings.vcodecsettings = self.vcodecsettings
+        else:
+            self.settings.vcodecsettings = {}
+
+        self.debug("Returning %s", self.settings)
+
+        return self.settings
+
 
 class ExportSettingsDialog(gtk.Dialog):
 
     def __init__(self, settings):
         gtk.Dialog.__init__(self, parent=None,
-                            title="Export Settings",
+                            title=_("Export settings"),
                             flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                      gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
@@ -332,6 +424,4 @@ class ExportSettingsDialog(gtk.Dialog):
         self.setwidget.show_all()
 
     def getSettings(self):
-        self.setwidget.updateSettings()
-        return self.setwidget.settings
-
+        return self.setwidget.updateSettings()
