@@ -20,18 +20,36 @@
 # Boston, MA 02111-1307, USA.
 
 import gobject
-import gtk
 import gst
+import gtk
 import pango
-import pitivi.instance as instance
-import dnd
-from pathwalker import PathWalker
-from filelisterrordialog import FileListErrorDialog
+import os
+import time
+
+from urllib import unquote
+from gettext import gettext as _
+from gettext import ngettext
+
+import pitivi.ui.dnd as dnd
+from pitivi.ui.pathwalker import PathWalker
+from pitivi.ui.filelisterrordialog import FileListErrorDialog
 from pitivi.configure import get_pixmap_dir
 from pitivi.signalgroup import SignalGroup
-from gettext import gettext as _
-from urllib import unquote
-import os
+from pitivi.stream import VideoStream, AudioStream, TextStream
+from pitivi.settings import GlobalSettings
+from pitivi.utils import beautify_length
+from pitivi.log.loggable import Loggable
+
+GlobalSettings.addConfigSection('clip-library')
+GlobalSettings.addConfigOption('lastImportFolder',
+    section='clip-library',
+    key='last-folder',
+    environment='PITIVI_IMPORT_FOLDER',
+    default=os.path.expanduser("~"))
+GlobalSettings.addConfigOption('closeImportDialog',
+    section='clip-library',
+    key='close-import-dialog-after-import',
+    default=True)
 
 (COL_ICON,
  COL_INFOTEXT,
@@ -42,10 +60,13 @@ import os
 ui = '''
 <ui>
     <menubar name="MainMenuBar">
-        <menu action="File">
+        <menu action="Library">
             <placeholder name="SourceList" >
                 <menuitem action="ImportSources" />
                 <menuitem action="ImportSourcesFolder" />
+                <menuitem action="RemoveSources" />
+                <separator />
+                <menuitem action="InsertEnd" />
             </placeholder>
         </menu>
     </menubar>
@@ -57,22 +78,64 @@ ui = '''
 </ui>
 '''
 
-def beautify_length(length):
-    """ Returns a string version of a nanoseconds value """
-    sec = length / gst.SECOND
-    mins = sec / 60
-    sec = sec % 60
-    if mins < 60:
-        return "%02dm%02ds" % (mins, sec)
-    hours = mins / 60
-    mins = mins % 60
-    return "%02dh%02dm%02ds" % (hours, mins, sec)
+INVISIBLE = gtk.gdk.pixbuf_new_from_file(os.path.join(get_pixmap_dir(), 
+    "invisible.png"))
 
-class SourceList(gtk.VBox):
+def beautify_stream(stream):
+
+    if type(stream) == AudioStream:
+        if stream.raw:
+            templ = ngettext("<b>Audio:</b> %d channel at %d <i>Hz</i> (%d <i>bits</i>)",
+                    "<b>Audio:</b> %d channels at %d <i>Hz</i> (%d <i>bits</i>)",
+                    stream.channels)
+            try:
+                templ = templ % (stream.channels, stream.rate, stream.width)
+            except TypeError:
+                import pdb; pdb.set_trace()
+            return templ
+
+        return _("<b>Unknown Audio format:</b> %s") % stream.audiotype
+
+    elif type(stream) == VideoStream:
+        if stream.raw:
+            if stream.framerate.num:
+                templ = _("<b>Video:</b> %d x %d <i>pixels</i> at %.2f<i>fps</i>")
+                templ = templ % (stream.par * stream.width , stream.height,
+                        float(stream.framerate))
+            else:
+                templ = _("<b>Image:</b> %d x %d <i>pixels</i>")
+                templ = templ % (stream.par * stream.width, stream.height)
+            return templ
+        return _("<b>Unknown Video format:</b> %s") % stream.videotype
+
+    elif type(stream) == TextStream:
+        return _("<b>Text:</b> %s") % stream.texttype
+
+    raise NotImplementedError
+
+def beautify_factory(factory):
+    ranks = {VideoStream: 0, AudioStream: 1}
+    def stream_sort_key(stream):
+        return ranks[type(stream)]
+
+    streams = factory.getOutputStreams()
+    streams.sort(key=stream_sort_key)
+    return ("<b>" + unquote(factory.displayname) + "</b>\n" +
+        "\n".join((beautify_stream(stream) for stream in streams)))
+
+class SourceList(gtk.VBox, Loggable):
     """ Widget for listing sources """
 
-    def __init__(self):
+    __gsignals__ = {
+        'play': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
+                (gobject.TYPE_PYOBJECT,))
+        }
+
+    def __init__(self, instance, uiman):
         gtk.VBox.__init__(self)
+        Loggable.__init__(self)
+
+        self.app = instance
 
         # Store
         # icon, infotext, objectfactory, uri, length
@@ -110,7 +173,6 @@ class SourceList(gtk.VBox):
 
         # import sources dialogbox
         self._importDialog = None
-        self._lastfolder = None
 
         # TreeView
         # Displays icon, name, type, length
@@ -121,6 +183,7 @@ class SourceList(gtk.VBox):
         self.treeview.set_headers_visible(False)
         tsel = self.treeview.get_selection()
         tsel.set_mode(gtk.SELECTION_MULTIPLE)
+        tsel.connect("changed", self._treeSelectionChanged)
 
         pixbufcol = gtk.TreeViewColumn(_("Icon"))
         pixbufcol.set_expand(False)
@@ -184,10 +247,10 @@ class SourceList(gtk.VBox):
         # Connect to project.  We must remove and reset the callbacks when
         # changing project.
         self.project_signals = SignalGroup()
-        self._connectToProject(instance.PiTiVi.current)
-        instance.PiTiVi.connect("new-project-loaded",
+        self._connectToProject(self.app.current)
+        self.app.connect("new-project-loaded",
             self._newProjectLoadedCb)
-        instance.PiTiVi.connect("new-project-failed",
+        self.app.connect("new-project-failed",
             self._newProjectFailedCb)
 
         # default pixbufs
@@ -200,9 +263,11 @@ class SourceList(gtk.VBox):
                            gtk.gdk.ACTION_COPY)
         self.connect("drag_data_received", self._dndDataReceivedCb)
 
-        self.treeview.drag_source_set(gtk.gdk.BUTTON1_MASK,
-                                      [dnd.URI_TUPLE, dnd.FILESOURCE_TUPLE],
-                                      gtk.gdk.ACTION_COPY)
+        self.treeview.drag_source_set(0,[], gtk.gdk.ACTION_COPY)
+        self.treeview.connect("motion-notify-event",
+            self._treeViewMotionNotifyEventCb)
+        self.treeview.connect("button-release-event",
+            self._treeViewButtonReleaseCb)
         self.treeview.connect("drag_begin", self._dndTreeBeginCb)
         self.treeview.connect("drag_data_get", self._dndDataGetCb)
 
@@ -212,18 +277,33 @@ class SourceList(gtk.VBox):
         # Error dialog box
         self.errorDialogBox = None
 
-        # our actions
+        # always available
         actions = (
             ("ImportSources", gtk.STOCK_ADD, _("_Import clips..."),
                 None, _("Import clips to use"), self._importSourcesCb),
             ("ImportSourcesFolder", gtk.STOCK_ADD,
-                _("_Import folder of clips..."), None,
+                _("Import _folder of clips..."), None,
                 _("Import folder of clips to use"), self._importSourcesFolderCb),
         )
-        self.actiongroup = gtk.ActionGroup("sourcelist")
-        self.actiongroup.add_actions(actions)
-        uiman = instance.PiTiVi.gui.uimanager
-        uiman.insert_action_group(self.actiongroup, 0)
+
+        # only available when selection is non-empty 
+        selection_actions = (
+            ("RemoveSources", gtk.STOCK_DELETE,
+                _("_Remove from project"), None, None,
+                self._removeSourcesCb),
+            ("InsertEnd", gtk.STOCK_COPY,
+                _("Insert at _end of timeline"), "Insert", None,
+                self._insertEndCb),
+        )
+
+        actiongroup = gtk.ActionGroup("sourcelistpermanent")
+        actiongroup.add_actions(actions)
+        uiman.insert_action_group(actiongroup, 0)
+
+        self.selection_actions = gtk.ActionGroup("sourcelistselection")
+        self.selection_actions.add_actions(selection_actions)
+        self.selection_actions.set_sensitive(False)
+        uiman.insert_action_group(self.selection_actions, 0)
         uiman.add_ui_from_string(ui)
 
     def _importSourcesCb(self, unused_action):
@@ -232,6 +312,19 @@ class SourceList(gtk.VBox):
     def _importSourcesFolderCb(self, unused_action):
         self.showImportSourcesDialog(True)
 
+    def _removeSourcesCb(self, unused_action):
+        self._removeSources()
+
+    def _insertEndCb(self, unused_action):
+        timeline = self.app.current.timeline
+        sources = self.app.current.sources
+        start = timeline.duration
+        for uri in self.getSelectedItems():
+            factory = sources[uri]
+            source = timeline.addSourceFactory(factory)
+            source.setStart(start)
+            start += source.duration
+
     def _getIcon(self, iconname, alternate):
         icontheme = gtk.icon_theme_get_default()
         pixdir = get_pixmap_dir()
@@ -257,7 +350,9 @@ class SourceList(gtk.VBox):
         self.project_signals.connect(
             project.sources, "file_removed", None, self._fileRemovedCb)
         self.project_signals.connect(
-            project.sources, "not_media_file", None, self._notMediaFileCb)
+            project.sources, "discovery-error", None, self._discoveryErrorCb)
+        self.project_signals.connect(
+            project.sources, "missing-plugins", None, self._missingPluginsCb)
         self.project_signals.connect(
             project.sources, "ready", None, self._sourcesStoppedImportingCb)
         self.project_signals.connect(
@@ -276,7 +371,7 @@ class SourceList(gtk.VBox):
         if displayed:
             if self.showingTreeView:
                 return
-            gst.debug("displaying tree view")
+            self.debug("displaying tree view")
             self.remove(self.textbox)
             self.txtlabel.hide()
             if usesignals:
@@ -290,7 +385,7 @@ class SourceList(gtk.VBox):
         else:
             if not self.showingTreeView:
                 return
-            gst.debug("hiding tree view")
+            self.debug("hiding tree view")
             self.remove(self.scrollwin)
             self.scrollwin.hide()
             self.pack_start(self.textbox)
@@ -300,7 +395,7 @@ class SourceList(gtk.VBox):
 
     def _dragMotionCb(self, unused_layout, unused_context, unused_x, unused_y,
                       unused_timestamp):
-        gst.log("motion")
+        self.log("motion")
         gobject.idle_add(self._displayTreeView, True, False)
 
     def showImportSourcesDialog(self, select_folders=False):
@@ -314,54 +409,63 @@ class SourceList(gtk.VBox):
         else:
             chooser_action = gtk.FILE_CHOOSER_ACTION_OPEN
             dialogtitle = _("Import a clip")
+        close_after = gtk.CheckButton(_("Close after importing files"))
+        close_after.set_active(self.app.settings.closeImportDialog)
 
         self._importDialog = gtk.FileChooserDialog(dialogtitle, None,
                                                    chooser_action,
                                                    (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE,
                                                     gtk.STOCK_ADD, gtk.RESPONSE_OK))
-
+        self._importDialog.props.extra_widget = close_after
         self._importDialog.set_default_response(gtk.RESPONSE_OK)
         self._importDialog.set_select_multiple(True)
         self._importDialog.set_modal(False)
-        if self._lastfolder:
-            self._importDialog.set_current_folder(self._lastfolder)
+        self._importDialog.set_current_folder(self.app.settings.lastImportFolder)
+
         self._importDialog.connect('response', self._dialogBoxResponseCb, select_folders)
         self._importDialog.connect('close', self._dialogBoxCloseCb)
         self._importDialog.show()
 
-    def addFiles(self, files):
+    def addUris(self, files):
         """ Add files to the list """
-        instance.PiTiVi.current.sources.addUris(files)
+        self.app.current.sources.addUris(files)
 
     def addFolders(self, folders):
         """ walks the trees of the folders in the list and adds the files it finds """
-        instance.PiTiVi.threads.addThread(PathWalker, folders, instance.PiTiVi.current.sources.addUris)
+        self.app.threads.addThread(PathWalker, folders, self.app.current.sources.addUris)
 
     def _addFactory(self, factory):
-        if factory.thumbnail:
+        video = factory.getOutputStreams(VideoStream)
+        if video and video[0].thumbnail:
+            thumbnail_file = video[0].thumbnail
             try:
-                gst.debug("attempting to open thumbnail file '%s'" % factory.thumbnail)
-                pixbuf = gtk.gdk.pixbuf_new_from_file(factory.thumbnail)
+                self.debug("attempting to open thumbnail file '%s'",
+                        thumbnail_file)
+                pixbuf = gtk.gdk.pixbuf_new_from_file(thumbnail_file)
             except:
-                gst.error("Failure to create thumbnail from file '%s'" % factory.thumbnail)
-                if factory.is_video:
-                    thumbnail = self.videofilepixbuf
-                elif factory.is_audio:
-                    thumbnail = self.audiofilepixbuf
+                self.error("Failure to create thumbnail from file '%s'",
+                        thumbnail_file)
+                thumbnail = self.videofilepixbuf
             else:
-                if not factory.video_info_stream:
-                    desiredheight = 64 * pixbuf.get_height() / pixbuf.get_width()
-                else:
-                    vi = factory.video_info_stream
-                    desiredheight = int(64 / float(vi.dar))
-                thumbnail = pixbuf.scale_simple(64, desiredheight, gtk.gdk.INTERP_BILINEAR)
+                desiredheight = int(64 / float(video[0].dar))
+                thumbnail = pixbuf.scale_simple(64,
+                        desiredheight, gtk.gdk.INTERP_BILINEAR)
         else:
-            thumbnail = factory.is_video and self.videofilepixbuf or self.audiofilepixbuf
+            if video:
+                thumbnail = self.videofilepixbuf
+            else:
+                thumbnail = self.audiofilepixbuf
+
+        if not factory.duration or factory.duration == gst.CLOCK_TIME_NONE:
+            duration = ''
+        else:
+            duration = beautify_length(factory.duration)
+
         self.storemodel.append([thumbnail,
-                                factory.getPrettyInfo(),
-                                factory,
-                                factory.name,
-                                factory.duration and "<b>%s</b>" % beautify_length(factory.duration) or ""])
+            beautify_factory(factory),
+            factory,
+            factory.name,
+            duration])
         self._displayTreeView()
 
     # sourcelist callbacks
@@ -381,10 +485,14 @@ class SourceList(gtk.VBox):
         if not len(model):
             self._displayTreeView(False)
 
-    def _notMediaFileCb(self, unused_sourcelist, uri, reason, extra):
+    def _discoveryErrorCb(self, unused_sourcelist, uri, reason, extra):
         """ The given uri isn't a media file """
         self.infostub.addErrors(uri, reason, extra)
 
+    def _missingPluginsCb(self, sourcelist, uri, details, descriptions):
+        #self.infostub.addErrors(uri, "Missing plugins", "\n".join(descriptions))
+        pass
+
     def _sourcesStartedImportingCb(self, unused_sourcelist):
         if not self.infostub.showing:
             self.pack_start(self.infostub, expand=False)
@@ -406,35 +514,32 @@ class SourceList(gtk.VBox):
         self.errorDialogBox.destroy()
         self.errorDialogBox = None
 
-
     ## Import Sources Dialog Box callbacks
 
     def _dialogBoxResponseCb(self, dialogbox, response, select_folders):
-        gst.debug("response:%r" % response)
+        self.debug("response:%r", response)
         if response == gtk.RESPONSE_OK:
-            self._lastfolder = dialogbox.get_current_folder()
+            lastfolder = dialogbox.get_current_folder()
+            self.app.settings.lastImportFolder = lastfolder
+            self.app.settings.closeImportDialog = \
+                dialogbox.props.extra_widget.get_active()
             filenames = dialogbox.get_uris()
             if select_folders:
                 self.addFolders(filenames)
             else:
-                self.addFiles(filenames)
+                self.addUris(filenames)
+            if self.app.settings.closeImportDialog:
+                dialogbox.destroy()
+                self._importDialog = None
         else:
             dialogbox.destroy()
             self._importDialog = None
 
     def _dialogBoxCloseCb(self, unused_dialogbox):
-        gst.debug("closing")
+        self.debug("closing")
         self._importDialog = None
 
-
-    ## UI Button callbacks
-
-    def _addButtonClickedCb(self, unused_widget=None):
-        """ called when a user clicks on the add button """
-        self.showImportSourcesDialog()
-
-    def _removeButtonClickedCb(self, unused_widget=None):
-        """ Called when a user clicks on the remove button """
+    def _removeSources(self):
         tsel = self.treeview.get_selection()
         if tsel.count_selected_rows() < 1:
             return
@@ -444,7 +549,17 @@ class SourceList(gtk.VBox):
         selected.sort(reverse=True)
         for path in selected:
             uri = model[path][COL_URI]
-            del instance.PiTiVi.current.sources[uri]
+            del self.app.current.sources[uri]
+
+    ## UI Button callbacks
+
+    def _addButtonClickedCb(self, unused_widget=None):
+        """ called when a user clicks on the add button """
+        self.showImportSourcesDialog()
+
+    def _removeButtonClickedCb(self, unused_widget=None):
+        """ Called when a user clicks on the remove button """
+        self._removeSources()
 
     def _playButtonClickedCb(self, unused_widget):
         """ Called when a user clicks on the play button """
@@ -454,25 +569,102 @@ class SourceList(gtk.VBox):
             return
         path = paths[0]
         factory = model[path][COL_FACTORY]
-        gst.debug("Let's play %s" % factory.name)
-        instance.PiTiVi.playground.playTemporaryFilesourcefactory(factory)
+        self.debug("Let's play %s", factory.name)
+        self.emit('play', factory)
+
+    _dragStarted = False
+    _dragButton = None
+    _dragX = 0
+    _dragY = 0
+    _ignoreRelease = False
+
+    def _rowUnderMouseSelected(self, treeview, event):
+        result = treeview.get_path_at_pos(int(event.x), int(event.y))
+        if result:
+            path = result[0]
+            return treeview.get_selection().path_is_selected(path)
+        return False
+
+    def _nothingUnderMouse(self, treeview, event):
+        return not bool(treeview.get_path_at_pos(int(event.x), int(event.y)))
+
+    def _treeViewButtonPressEventCb(self, treeview, event):
+        chain_up = True
 
-    def _treeViewButtonPressEventCb(self, unused_treeview, event):
         if event.button == 3:
             self.popup.popup(None, None, None, event.button, event.time)
+            chain_up = False
+
+        else:
+
+            if not event.state & (gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK):
+                chain_up = not self._rowUnderMouseSelected(treeview, event)
+
+            self._dragStarted = False
+            self._dragButton = event.button 
+            self._dragX = int(event.x)
+            self._dragY = int(event.y)
+
+        if chain_up:
+            gtk.TreeView.do_button_press_event(treeview, event)
+        else:
+            treeview.grab_focus()
+
+        self._ignoreRelease = chain_up
+
+        return True
+
+    def _treeViewMotionNotifyEventCb(self, treeview, event):
+        if not self._dragButton:
+            return True
+
+        if self._nothingUnderMouse(treeview, event):
+            return True
+
+        if treeview.drag_check_threshold(self._dragX, self._dragY,
+            int(event.x), int(event.y)):
+            context = treeview.drag_begin(
+                [dnd.URI_TUPLE, dnd.FILESOURCE_TUPLE],
+                gtk.gdk.ACTION_COPY,
+                self._dragButton,
+                event)
+            self._dragStarted = True
+        return False
+
+    def _treeViewButtonReleaseCb(self, treeview, event):
+        if event.button == self._dragButton:
+            self._dragButton = None
+            if (not self._ignoreRelease) and (not self._dragStarted):
+                treeview.get_selection().unselect_all()
+                result = treeview.get_path_at_pos(int(event.x), int(event.y))
+                if result:
+                    path = result[0]
+                    treeview.get_selection().select_path(path)
+        return False
+
+    def _treeSelectionChanged(self, tsel):
+        if self.getSelectedItems():
+            self.selection_actions.set_sensitive(True)
+        else:
+            self.selection_actions.set_sensitive(False)
 
     def _rowActivatedCb(self, unused_treeview, path, unused_column):
         factory = self.storemodel[path][COL_FACTORY]
-        instance.PiTiVi.playground.playTemporaryFilesourcefactory(factory)
+        self.emit('play', factory)
 
     def _newProjectLoadedCb(self, unused_pitivi, project):
         # clear the storemodel
         self.storemodel.clear()
         self._connectToProject(project)
         # synchronize the storemodel with the new project's sourcelist
-        for uri, factory in project.sources:
-            gst.log("loading uri %s" % uri)
-            self._addFactory(factory)
+        if project.loaded:
+            for uri, factory in project.sources:
+                self.log("loading uri %s", uri)
+                self._addFactory(factory)
+        else:
+            if not self.infostub.showing:
+                self.pack_start(self.infostub, expand=False)
+                self.infostub.startingImport()
 
     def _newProjectFailedCb(self, unused_pitivi, unused_reason,
         unused_uri):
@@ -494,7 +686,7 @@ class SourceList(gtk.VBox):
             # or it's on local system with "file://"
             return os.path.isfile(path)
 
-        gst.debug("targettype:%d, selection.data:%r" % (targettype, selection.data))
+        self.debug("targettype:%d, selection.data:%r", targettype, selection.data)
         directories = []
         if targettype == dnd.TYPE_URI_LIST:
             incoming = [unquote(x.strip('\x00')) for x in selection.data.strip().split("\r\n") if x.strip('\x00')]
@@ -508,35 +700,32 @@ class SourceList(gtk.VBox):
                 directories = [incoming]
         if directories:
             self.addFolders(directories)
-        self.addFiles(filenames)
+        self.addUris(filenames)
 
     def _dndTreeBeginCb(self, unused_widget, context):
-        gst.info("tree drag_begin")
+        self.info("tree drag_begin")
         model, paths = self.treeview.get_selection().get_selected_rows()
         if len(paths) < 1:
             context.drag_abort(int(time.time()))
         else:
             row = model[paths[0]]
-            self.treeview.drag_source_set_icon_pixbuf(row[COL_ICON])
+            context.set_icon_pixbuf(row[COL_ICON], 0, 0)
 
     def getSelectedItems(self):
         """ returns a list of selected items uri """
         model, rows = self.treeview.get_selection().get_selected_rows()
         return [model[path][COL_URI] for path in rows]
 
-    def _dndDataGetCb(self, unused_widget, unused_context, selection,
+    def _dndDataGetCb(self, unused_widget, context, selection,
                       targettype, unused_eventtime):
-        gst.info("data get, type:%d" % targettype)
+        self.info("data get, type:%d", targettype)
         uris = self.getSelectedItems()
         if len(uris) < 1:
             return
-        if targettype == dnd.TYPE_PITIVI_FILESOURCE:
-            selection.set(selection.target, 8,
-                          uris[0])
-        elif targettype == dnd.TYPE_URI_LIST:
-            selection.set(selection.target, 8,
-                          '\n'.join(uris))
-class InfoStub(gtk.HBox):
+        selection.set(selection.target, 8, '\n'.join(uris))
+        context.set_icon_pixbuf(INVISIBLE, 0, 0)
+
+class InfoStub(gtk.HBox, Loggable):
     """
     Box used to display information on the current state of the lists
     """
@@ -549,6 +738,7 @@ class InfoStub(gtk.HBox):
 
     def __init__(self):
         gtk.HBox.__init__(self)
+        Loggable.__init__(self)
         self.errors = []
         self.showing = False
         self._importingmessage = _("Importing clips...")
@@ -656,14 +846,12 @@ class InfoStub(gtk.HBox):
         self.emit("remove-me")
 
     def show(self):
-        gst.log("showing")
+        self.log("showing")
         self.show_all()
         self.showing = True
 
     def hide(self):
-        gst.log("hiding")
+        self.log("hiding")
         gtk.VBox.hide(self)
         self.showing = False
-
-
-
+gobject.type_register(SourceList)
